package eu.dnetlib.uoaauthorizationlibrary.utils;

import eu.dnetlib.uoaauthorizationlibrary.configuration.SecurityConfig;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.util.Collections;

@Component
public class AuthorizationUtils {
    private final Logger log = Logger.getLogger(this.getClass());
    private final SecurityConfig securityConfig;

    @Autowired
    AuthorizationUtils(SecurityConfig securityConfig) {
        this.securityConfig = securityConfig;
    }

    private String getToken(HttpServletRequest request) {
        if (request.getCookies() == null) {
            return null;
        }
        for (Cookie c : request.getCookies()) {
            if (c.getName().equals("AccessToken")) {
                return c.getValue();
            }

        }
        return null;
    }

    public UserInfo getUserInfo(HttpServletRequest request) {
        String url = securityConfig.getUserInfoUrl() + (securityConfig.isDeprecated()?getToken(request):"");
        RestTemplate restTemplate = new RestTemplate();
        try {
            ResponseEntity<UserInfo> response = restTemplate.exchange(url, HttpMethod.GET, createHeaders(request), UserInfo.class);
            return  response.getBody();
        } catch (RestClientException e) {
            log.error(e.getMessage());
            return null;
        }
    }

    private HttpEntity<HttpHeaders> createHeaders(HttpServletRequest request) {
        HttpHeaders headers = new HttpHeaders();
        headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));
        headers.set("Cookie", request.getHeader("Cookie"));
        return new HttpEntity<>(headers);
    }
}
